// This file was generated by ecm_setup_version(): DO NOT EDIT!

#ifndef MPVQT_VERSION_H
#define MPVQT_VERSION_H

#define MPVQT_VERSION_STRING "1.1.1"
#define MPVQT_VERSION_MAJOR 1
#define MPVQT_VERSION_MINOR 1
#define MPVQT_VERSION_PATCH 1
#define MPVQT_VERSION ((1<<16)|(1<<8)|(1))

#endif
